import os
from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.utils import simplejson as json, simplejson

from elections.models import Candidate, Election, BackgroundCategory, Background,\
                                BackgroundCandidate, PersonalData, PersonalDataCandidate,\
                                Category, Question, Answer
from elections.forms import CandidateUpdateForm, CandidateForm, CandidateLinkForm, BackgroundCandidateForm, PersonalDataCandidateForm, AnswerForm, CandidatePhotoForm

dirname = os.path.dirname(os.path.abspath(__file__))

PASSWORD = 'password'


class CandidateTagsTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(name='Joe', password=PASSWORD, email='joe@doe.cl')
        self.election = Election.objects.create(name='Foo', owner=self.user, slug='foo')
        self.candidate = Candidate.objects.create(name='John Candidate', election=self.election)
        self.personal_data = PersonalData.objects.create(election=self.election, label='foo')
        self.personal_data_candidate = PersonalDataCandidate.objects.create(personal_data=self.personal_data,
                                                                            candidate=self.candidate,
                                                                            value='bar')
        self.background_category = BackgroundCategory.objects.create(name='foo', election=self.election)
        self.background = Background.objects.create(name='foo', category=self.background_category)
        self.background_candidate = BackgroundCandidate.objects.create(value='var', background=self.background, candidate=self.candidate)

    def test_value_for_candidate_and_personal_data(self):
        self.candidate.associate_answer(self.answer)
        template = Template('{% load election_tags %}{% value_for_candidate_and_personal_data candidate personal_data %}')
        context = Context({"candidate": self.candidate, "personal_data": self.personal_data})
        self.assertEqual(template.render(context), self.personal_data_candidate.value)

    def test_value_for_candidate_and_background(self):
        self.candidate.associate_answer(self.answer)
        template = Template('{% load election_tags %}{% value_for_candidate_and_background candidate background %}')
        context = Context({"candidate": self.candidate, "background": self.background})
        self.assertEqual(template.render(context), self.background_candidate.value)


class CandidateModelTest(TestCase):
    def setUp(self):
        self.user, created = User.objects.get_or_create(username='joe')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')
        #deleting all background categories by default
        for backgroundcategory in self.election.backgroundcategory_set.all():
            backgroundcategory.delete()

    def test_create_candidate(self):
        candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            election=self.election)

        self.assertTrue(created)
        self.assertEqual(candidate.name, 'Juan Candidato')
        self.assertEqual(candidate.slug, 'juan-candidato')
        self.assertTrue(candidate.has_answered)
        self.assertEqual(candidate.election, self.election)

    def test_update_candidate(self):
        candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            election=self.election)

        candidate.name = 'nuevo_nombre'
        candidate.save()

        candidate2 = Candidate.objects.get(slug='juan-candidato', election=self.election)
        self.assertEqual(candidate2.name, 'nuevo_nombre')

    def test_create_two_candidate_with_same_election_with_same_name(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)

        self.assertRaises(IntegrityError, Candidate.objects.create,
                          name='Juan Candidato', election=self.election)

    def test_create_two_candidate_with_same_slug_in_different_election(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        election2, created = Election.objects.get_or_create(name='BarBaz2',
                                                           owner=self.user,
                                                           description='esta es una descripcion')
        candidate2 = Candidate.objects.create(name='Juan Candidato',
                                            election=election2)
        self.assertEqual(candidate.slug, candidate2.slug)

    def test_get_personal_data(self):
        candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            election=self.election)

        personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')
        personal_data_candidate, created = PersonalDataCandidate.objects.get_or_create(candidate=candidate,
                                                                                       personal_data=personal_data,
                                                                                       value='new_value')

        personal_data_set = candidate.get_personal_data
        self.assertTrue('foo' in personal_data_set)
        self.assertEqual('new_value', personal_data_set['foo'])

    def test_get_personal_data_with_no_values(self):
        candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
            election=self.election)

        personal_data, created = PersonalData.objects.get_or_create(election=self.election,
            label='foo')

        #And I will not create the value for that personal data
        personal_data_set = candidate.get_personal_data
        self.assertTrue('foo' in personal_data_set)
        self.assertTrue(personal_data_set['foo'] is None)

    def test_get_repeated_backgrounds(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        background_category = BackgroundCategory.objects.create(election=self.election,
                                                                    name='FooBar')
        background_category2 = BackgroundCategory.objects.create(election=self.election,
                                                                    name='FooBar')
        backgrounds = candidate.get_background
        self.assertEqual(len(backgrounds), 2)

    def test_get_background(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')
        background, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo')
        background2, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo2')
        background_category2, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar2')
        background3, created = Background.objects.get_or_create(category=background_category2,
                                                                name='foo3')


        background_data_candidate, created = BackgroundCandidate.objects.get_or_create(candidate=candidate,
                                                                                  background=background,
                                                                                  value="BarFoo")
        background_data_candidate2, created  = BackgroundCandidate.objects.get_or_create(candidate=candidate,
                                                                                  background=background2,
                                                                                  value="BarFoo2")
        background_data_candidate3, created  = BackgroundCandidate.objects.get_or_create(candidate=candidate,
                                                                                  background=background3,
                                                                                  value="BarFoo3")

        expected_dict = {
        1: {'name':'FooBar',
            'backgrounds': {
                1: {'name': 'foo', 'value': 'BarFoo'}, 
                2: {'name':'foo2', 'value': 'BarFoo2'}
                }
            },
        2: {'name': 'FooBar2',
            'backgrounds': {
                1: {'name':'foo3', 'value':'BarFoo3'}
                }
            }
        }
        self.assertEqual(candidate.get_background, expected_dict)
        
    def test_get_what_the_candidate_answered_when_has_been_answered(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')
        background, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo')
        background_data_candidate, created = BackgroundCandidate.objects.get_or_create\
                                            (candidate=candidate,\
                                            background=background,\
                                            value="BarFoo")
        what_the_candidate_answered = candidate.get_answer_for_background(background)
        expected_answer = "BarFoo"
        self.assertEqual(what_the_candidate_answered, expected_answer)
        
    def test_get_backgrounds_even_if_they_havent_been_answered_by_the_candidate(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')
        background, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo')
        background2, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo2')
        background_category2, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar2')
        background3, created = Background.objects.get_or_create(category=background_category2,
                                                                name='foo3')
                                                                                 
        expected_dict = {
        1: {'name':'FooBar',
            'backgrounds': {
                1: {'name': 'foo', 'value': None}, 
                2: {'name':'foo2', 'value': None}
                }
            },
        2: {'name': 'FooBar2',
            'backgrounds': {
                1: {'name':'foo3', 'value':None}
                }
            }
        }

        self.assertEqual(candidate.get_background, expected_dict)
        
        

    def test_add_personal_data(self):
        candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            election=self.election)

        personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')
        personal_data2, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo2')

        candidate.add_personal_data(personal_data, 'new_value')
        self.assertTrue('foo' in candidate.get_personal_data)
        self.assertEqual('new_value',candidate.get_personal_data['foo'])

        candidate.add_personal_data(personal_data, 'new_value2')

        self.assertTrue('foo' in candidate.get_personal_data)
        self.assertEqual('new_value2', candidate.get_personal_data['foo'])

        candidate.add_personal_data(personal_data2, 'new_value3')

        self.assertTrue('foo' in candidate.get_personal_data)
        self.assertEqual('new_value2', candidate.get_personal_data['foo'])
        self.assertTrue('foo2' in candidate.get_personal_data)
        self.assertEqual('new_value3', candidate.get_personal_data['foo2'])


    def test_add_background(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')
        background, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo')

        background2, created = Background.objects.get_or_create(category=background_category,
                                                                name='foo2')

        candidate.add_background(background, 'BarFoo')

        expected_dict = {
        1: {'name':'FooBar',
            'backgrounds': {
                1: {'name': 'foo', 'value': 'BarFoo'}, 
                2: {'name':'foo2', 'value': None}
                }
            },
        }
        self.assertEqual(candidate.get_background, expected_dict)

        candidate.add_background(background, 'BarFoo2')
        expected_dict = {
        1: {'name':'FooBar',
            'backgrounds': {
                1: {'name': 'foo', 'value': 'BarFoo2'}, 
                2: {'name':'foo2', 'value': None}
                }
            }
        }
        self.assertEqual(candidate.get_background, expected_dict)

        candidate.add_background(background2, 'BarFoo3')
        expected_dict = {
        1: {'name':'FooBar',
            'backgrounds': {
                1: {'name': 'foo', 'value': 'BarFoo2'}, 
                2: {'name':'foo2', 'value': 'BarFoo3'}
                }
            },
        }
        self.assertEqual(candidate.get_background, expected_dict)

    def test_get_questions_by_category(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=self.election,
                                                            slug='foo-cat')
        question, created = Question.objects.get_or_create(question='FooQuestion',
                                                            category=category)
        real_questions = candidate.get_questions_by_category(category)
        expected_questions = [question]
        self.assertEqual(real_questions[0].question, expected_questions[0].question)
        self.assertEqual(real_questions[0].category, expected_questions[0].category)

    def test_get_answer_by_question(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=self.election,
                                                            slug='foo-cat')
        question, created = Question.objects.get_or_create(question='FooQuestion',
                                                            category=category)
        another_question, created = Question.objects.get_or_create(question='BarQuestion',
                                                            category=category)
        answer, created = Answer.objects.get_or_create(question=question,
                                                        caption='BarAnswer1Question')
        candidate.associate_answer(answer)
        real_answer_1 = candidate.get_answer_by_question(question)
        real_answer_2 = candidate.get_answer_by_question(another_question)
        expected_answer_1 = answer
        expected_answer_2 = "no answer"
        self.assertEqual(real_answer_1, expected_answer_1)
        self.assertEqual(real_answer_2, expected_answer_2)

    def test_get_all_answers_by_category(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=self.election,
                                                            slug='foo-cat')
        question1, created = Question.objects.get_or_create(question='FooQuestion1',
                                                            category=category)
        question2, created = Question.objects.get_or_create(question='FooQuestion2',
                                                            category=category)
        answer1, created = Answer.objects.get_or_create(question=question1,
                                                        caption='BarAnswerQuestion1')
        answer2, created = Answer.objects.get_or_create(question=question2,
                                                        caption='BarAnswerQuestion2')
        candidate.associate_answer(answer1)
        candidate.associate_answer(answer2)
        expected_result = [(question1, answer1), (question2, answer2)]
        real_result = candidate.get_all_answers_by_category(category)

        self.assertEqual(real_result, expected_result)

    def test_get_answers_two_candidates(self):
        candidate1 = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        candidate2 = Candidate.objects.create(name='Mario Candidato',
                                            election=self.election)
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=self.election,
                                                            slug='foo-cat')
        question, created = Question.objects.get_or_create(question='FooQuestion',
                                                            category=category)
        answer1, created = Answer.objects.get_or_create(question=question,
                                                        caption='BarAnswer1Question')
        answer2, created = Answer.objects.get_or_create(question=question,
                                                        caption='BarAnswer2Question')
        candidate1.associate_answer(answer1)
        candidate2.associate_answer(answer2)
        real_result = [(question, answer1, answer2)]
        expected_result = candidate1.get_answers_two_candidates(candidate2, category)
        self.assertEqual(real_result, expected_result)


class CandidateDetailViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create(username='foobar')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.candidate = Candidate.objects.create(name='Juan Candidato',
                                                    election=self.election,
                                                    photo='photos/dummy.jpg')

    def test_detail_existing_candidate_view(self):
        response = self.client.get(reverse('candidate_detail',
                                            kwargs={
                                               'username': self.user.username,
                                               'election_slug': self.election.slug,
                                               'slug': self.candidate.slug}))

        self.assertEquals(response.status_code, 200)
        self.assertTrue('candidate' in response.context)
        self.assertEquals(response.context['candidate'], self.candidate)

    def test_detail_non_existing_candidate_view(self):
        response = self.client.get(reverse('candidate_detail',
                                           kwargs={
                                               'username': self.user.username,
                                               'election_slug': self.election.slug,
                                               'slug': 'asd-asd'}))
        self.assertEquals(response.status_code, 404)

    def test_detail_non_existing_candidate_for_user_view(self):
        election, created = Election.objects.get_or_create(name='AsdAsd',
                                                           owner=self.user,
                                                           slug='asdasd',
                                                           description='esta es una descripcion dos')
        response = self.client.get(reverse('candidate_detail',
                                           kwargs={
                                               'username': self.user.username,
                                               'election_slug': election.slug,
                                               'slug': self.candidate.slug}))
        self.assertEquals(response.status_code, 404)

        '''
        TODO: distintos usuarios, mismo slug de eleccion,
                mismo slug de candidato (inexistente en uno, pero existente en otra)
        '''


class CandidateCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

    def test_create_candidate_by_user_without_login(self):
        request = self.client.get(reverse('candidate_create', kwargs={'election_slug': self.election.slug}))

        self.assertEquals(request.status_code, 302)

    def test_create_candidate_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('candidate_create', kwargs={'election_slug': self.election.slug}))

        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], CandidateForm))
        self.assertFalse('has_answered' in request.context['form'].fields)
        self.assertTrue('election' in request.context)
        self.assertTrue(isinstance(request.context['election'], Election))

    def test_post_candidate_create_with_same_slug(self):
        candidate = Candidate.objects.create(name='Juan Candidato',
                                            election=self.election)
        self.client.login(username='joe', password='doe')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': candidate.name,
                  'photo': f,
                  'form-TOTAL_FORMS': u'0',
                  'form-INITIAL_FORMS': u'0',
                  'form-MAX_NUM_FORMS': u'',
                  'link-TOTAL_FORMS': u'0',
                  'link-INITIAL_FORMS': u'0',
                  'link-MAX_NUM_FORMS': u'',
                  }
        response = self.client.post(reverse('candidate_create', kwargs={'election_slug': self.election.slug}), params,
            follow=True)
        f.close()
        self.assertEquals(response.status_code, 200)

        # falta revisar que no tenga errores

    def test_post_candidate_create_without_login(self):
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'Juan Candidato', 'last_name': 'Candidato',
                  'slug': 'juan-candidato', 'photo': f,
                  'form-TOTAL_FORMS': u'0',
                  'form-INITIAL_FORMS': u'0',
                  'form-MAX_NUM_FORMS': u'',
                  'link-TOTAL_FORMS': u'0',
                  'link-INITIAL_FORMS': u'0',
                  'link-MAX_NUM_FORMS': u'',}
        response = self.client.post(reverse('candidate_create', kwargs={'election_slug': self.election.slug}), params)
        f.close()

        self.assertEquals(response.status_code, 302)

    def test_get_candidate_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('candidate_create',
                                    kwargs={'election_slug': 'strager_election_slug'}))
        self.assertEquals(response.status_code, 404)

    def test_post_candidate_create_with_login_stranger_election(self):
        user2 = User.objects.create_user(username='joe2', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        self.client.login(username='joe', password='doe')

        params = {'name': 'Juan Candidato', 'last_name': 'Candidato',
                  'slug': 'juan-candidato', 'photo': f,
                  'form-TOTAL_FORMS': u'0',
                  'form-INITIAL_FORMS': u'0',
                  'form-MAX_NUM_FORMS': u'',
                  'link-TOTAL_FORMS': u'0',
                  'link-INITIAL_FORMS': u'0',
                  'link-MAX_NUM_FORMS': u'',}
        response = self.client.post(reverse('candidate_create',
                                        kwargs={'election_slug': election2.slug}),
                                    params)
        f.close()

        self.assertEquals(response.status_code, 404)

    def test_if_the_request_is_sent_from_create_alone_it_redirects_to_it_create_alone(self):
        self.client.login(username='joe', password='doe')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'Juan Candidato',
                  'photo': f,
                  'form-TOTAL_FORMS': u'0',
                  'form-INITIAL_FORMS': u'0',
                  'form-MAX_NUM_FORMS': u'',
                  'link-TOTAL_FORMS': u'0',
                  'link-INITIAL_FORMS': u'0',
                  'link-MAX_NUM_FORMS': u'',}
        response = self.client.post(reverse('candidate_create_alone', kwargs={'election_slug': self.election.slug}), params, follow=True)
        f.seek(0)


        self.assertTemplateUsed(response, 'elections/updating/candidates.html')

    def test_post_candidate_create_logged(self):
        self.client.login(username='joe', password='doe')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'Juan Candidato',
                  'photo': f,
                  'form-TOTAL_FORMS': u'0',
                  'form-INITIAL_FORMS': u'0',
                  'form-MAX_NUM_FORMS': u'',
                  'link-TOTAL_FORMS': u'0',
                  'link-INITIAL_FORMS': u'0',
                  'link-MAX_NUM_FORMS': u'',}
        response = self.client.post(reverse('candidate_create', kwargs={'election_slug': self.election.slug}), params, follow=True)
        f.seek(0)


        self.assertEquals(response.status_code, 200)
        qs = Candidate.objects.filter(election= self.election, slug='juan-candidato')
        self.assertEquals(qs.count(), 1)
        candidate = qs.get()
        self.assertEquals(candidate.name, params['name'])
        # The CandidateForm doesnt contain the photo field
        # self.assertEquals(f.read(), candidate.photo.file.read())
        f.close()
        # os.unlink(candidate.photo.path)
        self.assertEquals(candidate.election, self.election)
        self.assertRedirects(response, reverse('candidate_create',
                                               kwargs={'election_slug': candidate.election.slug}))



    def test_renders_step_two_of_the_wizard(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('candidate_create', kwargs={'election_slug': self.election.slug}))
        self.assertTemplateUsed(response,'elections/wizard/step_two.html')

class CandidateUpdateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                            election=self.election)


    def test_update_candidate_by_user_without_login(self):
        request = self.client.get(reverse('candidate_update', kwargs={'slug': self.candidate.slug, 'election_slug': self.election.slug}))

        self.assertEquals(request.status_code, 302)

    def test_update_candidate_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('candidate_update', kwargs={'slug': self.candidate.slug, 'election_slug': self.election.slug}))

        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], CandidateUpdateForm))
        self.assertFalse('has_answered' in request.context['form'].fields)
        self.assertTrue('name' in request.context['form'].initial)
        self.assertEquals(request.context['form'].initial['name'], 'Juan Candidato')

    def test_post_candidate_update_without_login(self):
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'Juan Candidato', 'last_name': 'Candidato',
                  'photo': f,}
        response = self.client.post(reverse('candidate_update', kwargs={'slug': self.candidate.slug, 'election_slug': self.election.slug}), params)
        f.close()

        self.assertEquals(response.status_code, 302)

    def test_get_candidate_update_with_login_stranger_election(self):


        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz3',
                                                           description='esta es una descripcion')

        candidate2, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                            election=election2)

        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('candidate_update',
                                    kwargs={'slug': candidate2.slug, 'election_slug': election2.slug}))
        self.assertEquals(response.status_code, 404)

    def test_post_candidate_update_with_login_stranger_election(self):
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')


        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        candidate2, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                            election=election2)

        params = {'name': 'Juan Candidato', 'last_name': 'Candidato',
                  'photo': f,}

        self.client.login(username='joe', password='doe')
        response = self.client.post(reverse('candidate_update',
                                        kwargs={'slug': candidate2.slug, 'election_slug': election2.slug}),
                                    params)
        f.close()

        self.assertEquals(response.status_code, 404)

    def test_post_candidate_update_logged(self):
        self.client.login(username='joe', password='doe')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'Juanito', 'last_name': 'Candidato',
                  'photo': f,}
        response = self.client.post(reverse('candidate_update', kwargs={'slug': self.candidate.slug, 'election_slug': self.election.slug}), params, follow=True)
        f.seek(0)

        self.assertEquals(response.status_code, 200)
        qs = Candidate.objects.filter(election= self.election, slug='juan-candidato')
        self.assertEquals(qs.count(), 1)
        candidate = qs.get()
        self.assertEquals(candidate.name, params['name'])
        self.assertEquals(f.read(), candidate.photo.file.read())
        f.close()
        os.unlink(candidate.photo.path)
        self.assertEquals(candidate.election, self.election)
        self.assertRedirects(response, reverse('candidate_update',
                                               kwargs={'slug': self.candidate.slug, 'election_slug': candidate.election.slug}))


class CandidateUrlsTest(TestCase):
    def test_create_url(self):
        expected = '/bar-baz/candidate/create'
        result = reverse('candidate_create', kwargs={'election_slug': 'bar-baz'})
        self.assertEquals(result, expected)

    def test_detail_url(self):
        expected = '/juanito/bar-baz/juan-candidato'
        result = reverse('candidate_detail', kwargs={'username': 'juanito', 'election_slug': 'bar-baz', 'slug': 'juan-candidato'})
        self.assertEquals(result, expected)


class CandidateDataUpdateTest(TestCase):
     def setUp(self):
         self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
         self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz',
                                                            description='esta es una descripcion')

         self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                             election=self.election,
                                             photo='photos/dummy.jpg')

     def test_get_update_candidate_data_by_user_without_login(self):
         request = self.client.get(reverse('candidate_data_update', kwargs={'slug': self.candidate.slug, 'election_slug': self.election.slug}))

         self.assertEquals(request.status_code, 302)

     def test_get_update_candidate_data_by_user_success(self):
         self.client.login(username='joe', password='doe')
         request = self.client.get(reverse('candidate_data_update', kwargs={'slug': self.candidate.slug, 'election_slug': self.election.slug}))


         self.assertTrue('candidate' in request.context)
         self.assertEqual(request.context['candidate'], self.candidate)
         self.assertTrue('election' in request.context)
         self.assertEqual(request.context['election'], self.election)

     def test_update_candidate_data_strager_candidate(self):
         user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
         election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user2,
                                                            slug='barbaz2',
                                                            description='esta es una descripcion')

         candidate2, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                             election=election2,
                                             photo='photos/dummy.jpg')
         self.client.login(username='joe', password='doe')
         request = self.client.get(reverse('candidate_data_update', kwargs={'slug': candidate2.slug, 'election_slug': election2.slug}))
         
         self.assertEquals(request.status_code, 404)

     def test_post_candidate_data_update_405(self):
         self.client.login(username='joe', password='doe')
         request = self.client.post(reverse('candidate_data_update',
                                     kwargs={'slug': self.candidate.slug,
                                             'election_slug': self.election.slug}),
                                     {'var':'foo'})
         self.assertEquals(request.status_code, 405)


class AsyncDeleteCandidateTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                            election=self.election,
                                            photo='photos/dummy.jpg')


    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')
        url = reverse('async_delete_candidate')
        request = self.client.post(url , {'candidate_pk': self.candidate.pk})
        self.assertEquals(request.status_code, 200)

    def test_post_without_login(self):
        request = self.client.post(reverse('async_delete_candidate',
                                kwargs={}), {'candidate_pk': self.candidate.pk})
        self.assertEquals(request.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('async_delete_candidate',
                                kwargs={}),{'candidate_pk': self.candidate.pk})

        self.assertEquals(request.status_code, 405)

    def test_post_with_stranger_candidate(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        candidate2, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                            election=election2,
                                            photo='photos/dummy.jpg')

        self.client.login(username='joe', password='doe')
        url = reverse('async_delete_candidate')
        request = self.client.post(url,{'candidate_pk': candidate2.pk})

        self.assertEquals(request.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.candidate.pk
        request = self.client.post(reverse('async_delete_candidate'),
                                        {'candidate_pk': self.candidate.pk})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(Candidate.DoesNotExist, Candidate.objects.get, pk=temp_pk)


class CandidateCreateAjaxView(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='joe', email='joe@doe.cl')
        self.user2 = User.objects.create_user(username='doe', password='joe', email='dow@doe.cl')
        self.election = Election.objects.create(name='BarBaz', owner=self.user, slug='barbaz')
        self.url = reverse('async_create_candidate', kwargs={'election_slug': self.election.slug, })
        self.params = {'name': 'Juan Candidato'}

    def test_create_candidate_asynchronously(self):
        self.client.login(username=self.user.username, password='joe')
        response = self.client.post(self.url,self.params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.content, '{"result": "OK"}')



    def test_post_without_login(self):
        response = self.client.post(self.url, self.params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertRedirects(response, settings.LOGIN_URL + '?next=' + self.url)

    def test_post_to_not_owned_election(self):
        self.client.login(username=self.user2.username, password='joe')
        response = self.client.post(self.url, self.params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 404)

    def test_post_as_election_owner(self):
        self.client.login(username=self.user.username, password='joe')
        response = self.client.post(self.url, self.params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')

        self.assertEqual(response.status_code, 200)
        candidate = Candidate.objects.filter(election=self.election)

        self.assertEqual(candidate.count(), 1)
        candidate = candidate.get()
        self.assertEqual(candidate.name, self.params['name'])
        self.assertEqual(response.content, '{"result": "OK"}')
        self.assertEqual(response['Content-Type'], 'application/json')

    def test_post_as_election_owner_with_same_name(self):
        self.client.login(username=self.user.username, password='joe')
        candidate, created = Candidate.objects.get_or_create(election=self.election, name=self.params['name'])

        response = self.client.post(self.url, self.params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        candidate = Candidate.objects.filter(election=self.election)
        self.assertEqual(candidate.count(), 1)

        self.assertTrue('error' in simplejson.loads(response.content))
        self.assertEqual(response['Content-Type'], 'application/json')

    def test_post_invalid_answer(self):
        self.client.login(username=self.user.username, password='joe')
        params = {}

        response = self.client.post(self.url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        candidate = Candidate.objects.filter(election=self.election)
        self.assertEqual(candidate.count(), 0)

        self.assertTrue('error' in simplejson.loads(response.content))
        self.assertEqual(response['Content-Type'], 'application/json')

    def test_get_no_ajax(self):
        self.client.login(username=self.user.username, password='joe')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 400)

PASSWORD = 'doe'
class CandidateUpdateDataViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='asd@asd.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                          owner=self.user,
                                                          slug='barbaz',
                                                          description='esta es una descripcion')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                                 slug='juan-candidato',
                                                                 election=self.election)
        self.url = reverse('candidate_data_update', kwargs={'election_slug': self.election.slug, 'slug': self.candidate.slug})

    def test_get_not_logged(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, settings.LOGIN_URL + '?next=' + self.url)

    def test_get_not_owned_election(self):
        stranger_user = User.objects.create_user(username='John', password=PASSWORD, email='john@doe.cl')
        self.client.login(username=stranger_user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 404)

    def test_get_non_existing_candidate(self):
        url = reverse('candidate_data_update', kwargs={'election_slug': self.election.slug, 'slug': 'non_exist'})
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_owned_candidate(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertTemplateUsed(response, 'elections/updating/answers.html')
        self.assertEqual(response.status_code, 200)
        self.assertTrue('election' in response.context)
        self.assertTrue('candidate' in response.context)
        self.assertEqual(response.context['election'], self.election)
        self.assertEqual(response.context['candidate'], self.candidate)


        self.assertTrue('candidate_link_form' in response.context)
        self.assertIsInstance(response.context['candidate_link_form'], CandidateLinkForm)

        self.assertTrue('background_candidate_form' in response.context)
        self.assertIsInstance(response.context['background_candidate_form'], BackgroundCandidateForm)

        self.assertTrue('personal_data_candidate_form' in response.context)
        self.assertIsInstance(response.context['personal_data_candidate_form'], PersonalDataCandidateForm)

        self.assertTrue('answer_form' in response.context)
        self.assertIsInstance(response.context['answer_form'], AnswerForm)


    def test_create_a_link_asynchronously(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        link_name = 'mi link'
        link_url = 'http://milink.com'
        link_details = {
            'link_name': link_name,
            'link_url': link_url,
            'election_slug': self.election.slug,
            'candidate_slug': self.candidate.slug,
            'candidate_name': self.candidate.name,
        }
        url = reverse('link_create_ajax', kwargs={
            'candidate_pk': self.candidate.pk,
            })
        request = self.client.post(url, link_details)
        self.assertEquals(request.status_code, 200)
        self.assertEqual(self.candidate.link_set.count(), 1)
        link = self.candidate.link_set.all()[0]
        self.assertEqual(link.name, link_name)
        self.assertEqual(link.url, link_url)


    def test_post_failed(self):
        self.client.login(username='joe', password='doe')
        link_details = {
            'link_name': "",
            'link_url': "",
            'election_slug': self.election.slug,
            'candidate_slug': self.candidate.slug,
            'candidate_name': self.candidate.name,
        }
        url = reverse('link_create_ajax', kwargs={
            'candidate_pk': self.candidate.pk,
            })
        request = self.client.post(url, link_details)
        self.assertEqual(request.status_code, 200)
        self.assertEqual(self.candidate.link_set.count(), 0)
        body = simplejson.loads(request.content)
        self.assertTrue('name' in body["errors"])
        self.assertTrue('url' in body["errors"])

class MultipleCandidateDataUpdate(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='asd@asd.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
            owner=self.user,
            slug='barbaz',
            description='esta es una descripcion')

        self.first_candidate, created = Candidate.objects.get_or_create(name='Primer Candidato',
            slug='primer-candidato',
            election=self.election)

        self.second_candidate, created = Candidate.objects.get_or_create(name='Segundo Candidato',
            slug='segundo-candidato',
            election=self.election)

        self.url = reverse('multiple_candidate_data_update', kwargs={'election_slug': self.election.slug})

    def test_get_right_after_filling_their_names_and_showing_a_success_text(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        url = reverse('multiple_candidate_data_update_first_time', kwargs={'election_slug': self.election.slug})
        response = self.client.get(url)
        
        self.assertTemplateUsed(response, 'elections/updating/answers.html')
        self.assertContains(response, '<div class="success">')
        


    def test_get_first_candidate_ordered_by_name(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertTemplateUsed(response, 'elections/updating/answers.html')
        self.assertEqual(response.status_code, 200)


        self.assertTrue('election' in response.context)
        self.assertTrue('candidate' in response.context)

        self.assertEqual(response.context['election'], self.election)
        self.assertEqual(response.context['candidate'], self.first_candidate)


#This correspond to bug 191
class MultipleCandidateDataUpdateWith0Candidates(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='asd@asd.cl')
        self.election, created = Election.objects.get_or_create(name='NoCandidatesBarBaz',
            owner=self.user,
            slug='nocandidatesbarbaz',
            description='this election has no candidates')

        self.url = reverse('multiple_candidate_data_update', kwargs={'election_slug': self.election.slug})

    def test_redirect_to_add_new_users_when_candidate_list_is_empty(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 302)


    def test_show_create_new_candidate_html_is_shown(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url, follow=True)


        self.assertTemplateUsed(response, 'elections/updating/candidates.html')


class CandidateUpdatePhotoViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@exmaple.net')
        self.election = Election.objects.create(name='election', slug='election', owner=self.user)
        self.candidate = Candidate.objects.create(name='candidate', election=self.election)
        self.file = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        self.url = reverse('update_candidate_photo', kwargs={'pk': self.candidate.pk})

    def test_get_not_logged(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, settings.LOGIN_URL + '?next=' + self.url)

    def test_post_not_logged(self):
        response = self.client.post(self.url)
        self.assertRedirects(response, settings.LOGIN_URL + '?next=' + self.url)
    
    def test_get_not_owner(self):
        not_user = User.objects.create_user(username='doe', password=PASSWORD, email='doe@example.net')
        self.client.login(username=not_user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_post_not_owner(self):
        not_user = User.objects.create_user(username='doe', password=PASSWORD, email='doe@example.net')
        self.client.login(username=not_user.username, password=PASSWORD)
        response = self.client.post(self.url)
        self.assertEqual(response.status_code, 403)
    
    def test_get_owner(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'elections/candidate_photo_form.html')
        self.assertTrue('form' in response.context)
        self.assertIsInstance(response.context['form'], CandidatePhotoForm)
        self.assertTrue('candidate' in response.context)
        self.assertEqual(response.context['candidate'], self.candidate)
    
    def test_post_owner(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        params = {
            'photo': self.file
        }
        response = self.client.post(self.url, params)
        self.file.seek(0)
        self.assertRedirects(response, reverse('candidate_data_update', 
                                       kwargs={'election_slug': self.election.slug, 'slug': self.candidate.slug}))
        candidate = Candidate.objects.get(pk=self.candidate.pk)
        self.assertEqual(candidate.photo.file.read(), self.file.read())
        os.unlink(candidate.photo.path)


class CandidateListAjaxViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@exmaple.net')
        self.election = Election.objects.create(name='election', slug='election', owner=self.user)
        self.candidate_one = Candidate.objects.create(name='candidate one', election=self.election)
        self.candidate_two = Candidate.objects.create(name='candidate two', election=self.election)

    def test_get_list_for_a_defined_election(self):
        url_params = {'username':self.user.username,'election_slug':self.election.slug}
        url = reverse('candidate_list_json',kwargs=url_params)
        request_params = {}
        response = self.client.post(url,request_params,HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        candidate_names = json.loads(response.content)
        self.assertTrue(candidate_names.__len__(),2)
        self.assertEquals(candidate_names[0],{'name':'candidate one','id':self.candidate_one.pk})
        self.assertEquals(candidate_names[1],{'name':'candidate two','id':self.candidate_two.pk})





# -*- coding: utf-8 -*-


from django.test import TestCase
from elections.management.commands.elections_loader import *
from django.contrib.auth.models import User
from django.conf import settings
from elections.models import InformationSource


class QuestionsParserTestCase(TestCase):
    def setUp(self):
        self.lines = [
            ["category","la categoria1"],
            ["question","la pregunta1"],
            ["answer","respuesta 1"],
            ["answer","respuesta 2"],
            ["category","la categoria2"],
            ["question","la pregunta2"],
            ["answer","respuesta 3"],
            ["answer","respuesta 4"],
            ["answer","respuesta 5"],
            ["background history category","Background category 1"],
            ["background history","background history record 1"],
            ["background history","background history record 2"],
            ["background history category","Background category 2"],
            ["background history","background history record 3"],
            ["background history","background history record 4"],
            ["personal data","Party"],
            ["personal data","Age"],
        ]
        self.user = User.objects.create_user(username='ciudadanointeligente',
                                                password='fci',
                                                email='fci@ciudadanointeligente.cl')
        self.election = Election.objects.create(owner=self.user, name="Elección para molestar a la Fiera")
        self.election.category_set.all().delete()




    def test_create_categories(self):
        parser = QuestionsParser(self.election)
        parser.createQuestions(self.lines)
        election_after_questions_created = Election.objects.get(pk=self.election.pk)
        
        self.assertEquals(election_after_questions_created.category_set.count(), 2 )
        self.assertEquals(election_after_questions_created.category_set.all()[0].name, u"la categoria1")
        self.assertEquals(election_after_questions_created.category_set.all()[1].name, u"la categoria2")

    def test_create_questions(self):
        parser = QuestionsParser(self.election)
        parser.createQuestions(self.lines)
        election_after_questions_created = Election.objects.get(pk=self.election.pk)

        first_category_questions = election_after_questions_created.category_set.all()[0].question_set.all()

        self.assertEquals(first_category_questions.count(), 1)
        self.assertEquals(first_category_questions[0].question, u"la pregunta1")

        second_category_questions = election_after_questions_created.category_set.all()[1].question_set.all()

        self.assertEquals(second_category_questions.count(), 1)
        self.assertEquals(second_category_questions[0].question, u"la pregunta2")


    def test_create_answer(self):
        parser = QuestionsParser(self.election)
        parser.createQuestions(self.lines)
        election = Election.objects.get(pk=self.election.pk)

        first_category_questions = election.category_set.all()[0].question_set.all()
        second_category_questions = election.category_set.all()[1].question_set.all()

        self.assertEquals(first_category_questions[0].answer_set.count(), 2)
        self.assertEquals(first_category_questions[0].answer_set.all()[0].caption, u"respuesta 1")
        self.assertEquals(first_category_questions[0].answer_set.all()[1].caption, u"respuesta 2")
        self.assertEquals(second_category_questions[0].answer_set.count(), 3)
        self.assertEquals(second_category_questions[0].answer_set.all()[0].caption, u"respuesta 3")
        self.assertEquals(second_category_questions[0].answer_set.all()[1].caption, u"respuesta 4")
        self.assertEquals(second_category_questions[0].answer_set.all()[2].caption, u"respuesta 5")


    def test_create_background_category(self):
        parser = QuestionsParser(self.election)
        parser.createQuestions(self.lines)
        election_after_questions_created = Election.objects.get(pk=self.election.pk)
        self.assertEquals(election_after_questions_created.backgroundcategory_set.count(), 2)
        self.assertEquals(election_after_questions_created.backgroundcategory_set.all()[0].name, u"Background category 1")
        self.assertEquals(election_after_questions_created.backgroundcategory_set.all()[1].name, u"Background category 2")


    def test_create_background(self):
        parser = QuestionsParser(self.election)
        parser.createQuestions(self.lines)
        election_after_questions_created = Election.objects.get(pk=self.election.pk)

        first_background_category = election_after_questions_created.backgroundcategory_set.all()[0]
        second_background_category = election_after_questions_created.backgroundcategory_set.all()[1]


        self.assertEquals(first_background_category.background_set.count(), 2)
        self.assertEquals(first_background_category.background_set.all()[0].name, u"background history record 1")
        self.assertEquals(first_background_category.background_set.all()[1].name, u"background history record 2")
        self.assertEquals(second_background_category.background_set.count(), 2)
        self.assertEquals(second_background_category.background_set.all()[0].name, u"background history record 3")
        self.assertEquals(second_background_category.background_set.all()[1].name, u"background history record 4")

    def test_create_personal_data(self):
        parser = QuestionsParser(self.election)
        parser.createQuestions(self.lines)
        election_after_questions_created = Election.objects.get(pk=self.election.pk)

        self.assertEquals(election_after_questions_created.personaldata_set.count(), 2)
        self.assertEquals(election_after_questions_created.personaldata_set.all()[0].label, u"Party")
        self.assertEquals(election_after_questions_created.personaldata_set.all()[1].label, u"Age")

class QuestionsConflictingWithDefaultQuestions(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='ciudadanointeligente',
                                                password='fci',
                                                email='fci@ciudadanointeligente.cl')
        settings.DEFAULT_QUESTIONS = [{
            'Category':u'Educación',
            'Questions':[
                {
                    'question':u'¿Crees que Chile debe tener una educación gratuita?',
                    'answers':[u'Sí',u'No']
                },
                {
                    'question':u'¿Estas de acuerdo con la desmunicipalización?',
                    'answers':[u'Sí',u'No']
                }
            ]
        }]
        self.question_lines = [
            ["category","Educación"],
            ["question","¿Crees que Chile debe tener una educación gratuita?"],
            ["answer","Sí"],
            ["answer","No"],
            ["question","¿Estas de acuerdo con la desmunicipalización?"],
            ["answer","Sí"],
            ["answer","No"],
        ]
        self.line0 = ["", "",] #this line defines the type of the following elements
        self.line1 = [
                        "election", 
                        "candidate"
                        ] #this is a background record
        
        #the rest of the lines are going to be interpreted as answers
        self.line2 = ["Algarrobo", "Fiera Feroz"]
        self.line3 = ["Algarrobo", "Mickey"]

        self.lines = [self.line0, self.line1, self.line2, self.line3]
        self.styles = u"un estilo"

    def test_it_does_not_conflict_with_any_preexisting_category(self):
        
        self.loader = AnswersLoader('ciudadanointeligente', self.lines, self.question_lines, self.styles)
        self.loader.process()

        new_election = Election.objects.create(owner=self.user, name="Una elección nueva")
        parser = QuestionsParser(new_election)
        parser.createQuestions(self.lines)
        election_after_questions_created = Election.objects.get(pk=new_election.pk)
        

        self.assertEquals(election_after_questions_created.category_set.count(), 1 )
        self.assertEquals(election_after_questions_created.category_set.all()[0].name, u"Educación")


class CandidateLoaderTestCase(TestCase):
    def setUp(self):
        #The first line of the csv file defines how the rest of the file is
        #going to be read

        self.questions_lines = [
                ["category","la categoria1"],
                ["question","la pregunta1"],
                ["answer","respuesta 1"],
                ["answer","respuesta 2"],
                ["category","la categoria2"],
                ["question","la pregunta2"],
                ["answer","respuesta 3"],
                ["answer","respuesta 4"],
                ["answer","respuesta 5"],
                ["background history category","Background category 1"],
                ["background history","background history record 1"],
                ["background history","background history record 2"],
                ["background history category","Background category 2"],
                ["background history","background history record 3"],
                ["background history","background history record 4"],
                ["background history","first job"],
                ["personal data","Party"],
                ["personal data","Age"],
            ]
        self.line0 = ["", "", "personal data", "personal data", "background history", "link", "link", "answer", "answer"] #this line defines the type of the following elements
        self.line1 = [
                        "election", 
                        "candidate", 
                        "Party", #this is a personal data object
                        "Age", #this is a personal data object
                        "Background category 2 - first job",
                        "twitter",
                        "facebook",
                        "la pregunta1",
                        "la pregunta2"
                        ] #this is a background record
        
        #the rest of the lines are going to be interpreted as answers
        self.line2 = ["Algarrobo", "Fiera Feroz", "Partido Feroz", "2 años", "Seguradad en FCI","fieraferoz","http://facebook.com/fieraferoz"]
        self.line3 = ["Algarrobo", "Mickey", "Partido Ratón", "2 semanas", "Ratón inteligente en FCI", "ratoninteligente", "http://facebook.com/ratoninteligente"]

        self.lines = [self.line0, self.line1, self.line2, self.line3]
        self.user = User.objects.create_user(username='ciudadanointeligente',
                                                password='fci',
                                                email='fci@ciudadanointeligente.cl')
        self.styles = u"un estilo"


        self.loader = AnswersLoader('ciudadanointeligente', self.lines, self.questions_lines, self.styles)
        Election.objects.all().delete()


    

    def test_get_definitions_from_first_line(self):
        expected_definitions = {
            2 : {"label":"Party","type":"personal data"},
            3 : {"label":"Age","type":"personal data"},
            4 : {"label":"Background category 2 - first job","type":"background history"},
            5 : {"label":"twitter","type":"link"},
            6 : {"label":"facebook","type":"link"},
            7 : {"label":"la pregunta1","type":"answer"},
            8 : {"label":"la pregunta2","type":"answer"},
        }

        self.assertEquals(self.loader.definitions, expected_definitions)


    def test_creates_one_election(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_election = Election.objects.all()[0]
        self.assertEquals(the_election.name, u"Algarrobo")
        self.assertEquals(the_election.custom_style, self.styles)

    def test_create_only_one_election_with_the_same_name(self):
        the_election_1 = self.loader.get_election(self.line2, self.questions_lines)
        the_election_2 = self.loader.get_election(self.line3, self.questions_lines)
        self.assertEquals(Election.objects.filter(owner=self.user).count(), 1)

    def test_processes_the_questions(self):
        the_election_1 = self.loader.get_election(self.line2, self.questions_lines)
        self.assertEquals(the_election_1.personaldata_set.count(), 2)
        self.assertEquals(the_election_1.personaldata_set.all()[0].label, u"Party")
        self.assertEquals(the_election_1.personaldata_set.all()[1].label, u"Age")

    def test_only_assigns_style_to_newly_created_elections(self):
        the_election_1 = self.loader.get_election(self.line2, self.questions_lines)
        
        the_same_election_with_a_new_candidate = self.loader.get_election(self.line3, self.questions_lines)
        self.assertEquals(the_same_election_with_a_new_candidate.custom_style, self.styles)

    def test_get_candidate(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_candidate = self.loader.get_candidate(self.line2, the_election)

        self.assertEquals(the_candidate.name, u"Fiera Feroz")

    def test_it_create_two_candidates(self):
        self.loader.process()
        the_election = Election.objects.all()[0]
        self.assertEquals(the_election.candidate_set.count(), 2)
        self.assertEquals(the_election.candidate_set.all()[0].name, u"Fiera Feroz")

    def test_it_creates_the_questions(self):
        self.loader.process()
        the_election = Election.objects.all()[0]
        self.assertEquals(the_election.personaldata_set.count(), 2)
        self.assertEquals(the_election.personaldata_set.all()[0].label, u"Party")
        self.assertEquals(the_election.personaldata_set.all()[1].label, u"Age")
        #I'm being lazy and assuming that the questionparser is executed

    def test_it_assingns_personal_data(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_candidate = self.loader.get_candidate(self.line2, the_election)
        self.loader.assign_values(the_election, the_candidate, self.line2)

        self.assertEquals(the_election.personaldata_set.all()[0].personaldatacandidate_set.all()[0].value, u"Partido Feroz")
        self.assertEquals(the_election.personaldata_set.all()[1].personaldatacandidate_set.all()[0].value, u"2 años")

    def test_it_assigns_links(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_candidate = self.loader.get_candidate(self.line2, the_election)
        self.loader.assign_values(the_election, the_candidate, self.line2)
        self.assertEquals(the_candidate.link_set.count(), 2)


    def test_it_assigns_twitter(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_candidate = self.loader.get_candidate(self.line2, the_election)
        self.loader.assign_values(the_election, the_candidate, self.line2)

        self.assertEquals(the_candidate.link_set.all()[0].url, u"https://twitter.com/fieraferoz")
        self.assertEquals(the_candidate.link_set.all()[0].name, u"@fieraferoz")

    def test_it_assigns_facebook(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_candidate = self.loader.get_candidate(self.line2, the_election)
        self.loader.assign_values(the_election, the_candidate, self.line2)

        self.assertEquals(the_candidate.link_set.all()[1].url, u"http://facebook.com/fieraferoz")
        self.assertEquals(the_candidate.link_set.all()[1].name, u"Fiera Feroz")


    def test_it_assigns_background_value(self):
        the_election = self.loader.get_election(self.line2, self.questions_lines)
        the_candidate = self.loader.get_candidate(self.line2, the_election)
        self.loader.assign_values(the_election, the_candidate, self.line2)

        the_background = the_candidate.backgroundcandidate_set.count()
        self.assertEquals(the_candidate.backgroundcandidate_set.count(), 1)
        self.assertEquals(the_candidate.backgroundcandidate_set.all()[0].value, u"Seguradad en FCI")


    def test_it_assigns_backgrounds_and_personal_data(self):
        self.loader.process()

        the_election = Election.objects.all()[0]
        self.assertEquals(the_election.personaldata_set.all()[0].personaldatacandidate_set.all()[0].value, u"Partido Feroz")
        self.assertEquals(the_election.personaldata_set.all()[1].personaldatacandidate_set.all()[0].value, u"2 años")

        the_candidate = Candidate.objects.get(name="Fiera Feroz")
        the_background = the_candidate.backgroundcandidate_set.count()
        self.assertEquals(the_candidate.backgroundcandidate_set.count(), 1)
        self.assertEquals(the_candidate.backgroundcandidate_set.all()[0].value, u"Seguradad en FCI")



class AssociateAnswersWithCandidatesTestCase(TestCase):
    def setUp(self):
        #The first line of the csv file defines how the rest of the file is
        #going to be read

        self.questions_lines = [
                ["category","la categoria1"],
                ["question","la pregunta1"],
                ["answer","respuesta 1"],
                ["answer","respuesta 2"],
                ["category","la categoria2"],
                ["question","la pregunta2"],
                ["answer","respuesta 3"],
                ["answer","respuesta 4"],
                ["answer","respuesta 5"],
                ["question","la pregunta3"],
                ["answer","respuesta 6"],
                ["answer","respuesta 6"],
                ["background history category","Background category 1"],
                ["background history","background history record 1"],
                ["background history","background history record 2"],
                ["background history category","Background category 2"],
                ["background history","background history record 3"],
                ["background history","background history record 4"],
                ["background history","first job"],
                ["personal data","Party"],
                ["personal data","Age"],
            ]
        self.line0 = ["", "", "personal data", "personal data", "background history", "link", "link", "link", "answer", "answer", "answer"] #this line defines the type of the following elements
        self.line1 = [
                        "election", 
                        "candidate", 
                        "Party", #this is a personal data object
                        "Age", #this is a personal data object
                        "Background category 2 - first job",
                        "Link Personal",
                        "twitter",
                        "facebook",
                        "la pregunta1",
                        "la pregunta2",
                        "la pregunta3"
                        ] #this is a background record
        
        #the rest of the lines are going to be interpreted as answers
        self.line2 = [
        "Algarrobo", 
        "Fiera Feroz", 
        "Partido Feroz", 
        "2 años", 
        "Seguradad en FCI",
        "http://fieraferoz.com",
        "fieraferoz",
        "http://facebook.com/fieraferoz", 
        "respuesta 1<elmostrador 25 de diciembre en http://elmostrador.cl>",
        "respuesta 3 <elmostrador 25 de diciembre en http://elmostrador.cl>"]
        self.line3 = [
        "Algarrobo", 
        "Mickey", 
        "Partido Ratón", 
        "2 semanas", 
        "Ratón inteligente en FCI", 
        "",
        "ratoninteligente", 
        "http://facebook.com/ratoninteligente", 
        "respuesta 2  ",
        "asdf",
        "respuesta 6"]

        self.lines = [self.line0, self.line1, self.line2, self.line3]
        self.user = User.objects.create_user(username='ciudadanointeligente',
                                                password='fci',
                                                email='fci@ciudadanointeligente.cl')
        self.styles = u"un estilo"


        self.loader = AnswersLoader('ciudadanointeligente', self.lines, self.questions_lines, self.styles)
        Election.objects.all().delete()


    def test_it_associates_an_answer_to_the_questions(self):
        self.loader.process()
        fiera = Candidate.objects.get(name="Fiera Feroz")
        mickey = Candidate.objects.get(name="Mickey")
        self.assertEquals(fiera.answers.count(), 2)
        self.assertEquals(mickey.answers.count(), 1)
        self.assertTrue(fiera.answers.filter(caption="respuesta 1").exists())
        self.assertTrue(fiera.answers.filter(caption="respuesta 3").exists())
        self.assertTrue(mickey.answers.filter(caption="respuesta 2").exists())
        self.assertFalse(mickey.answers.filter(question__question="la pregunta2").exists())
        self.assertFalse(fiera.answers.filter(question__question="la pregunta3").exists())
        self.assertFalse(mickey.answers.filter(question__question="la pregunta3").exists())


    def test_it_creates_an_information_source_for_that_answer(self):
        self.loader.process()
        fiera = Candidate.objects.get(name="Fiera Feroz")
        question = Question.objects.get(question='la pregunta1')
        self.assertTrue(InformationSource.objects.filter(Q(question=question) & Q(candidate=fiera)).exists())
        information_source = InformationSource.objects.get(Q(question=question) & Q(candidate=fiera))
        self.assertEquals(information_source.content, u'elmostrador 25 de diciembre en http://elmostrador.cl')


    def test_it_creates_any_link_for_a_candidate(self):
        self.loader.process()
        fiera = Candidate.objects.get(name="Fiera Feroz")
        self.assertTrue(Link.objects.filter(candidate=fiera).filter(name="Link Personal").filter(url="http://fieraferoz.com").exists())

    def test_it_does_not_create_a_link_if_value_is_empty(self):
        self.loader.process()
        mickey = Candidate.objects.get(name="Mickey")
        self.assertFalse(Link.objects.filter(candidate=mickey).filter(name="Link Personal").exists())


    def test_multiple_questions_bug(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user,
                                                            slug='barbaz')
        category, created = Category.objects.get_or_create(name='FooCat',
                                                           slug='foocat',
                                                            election=election)
        question = Question.objects.create(question='la pregunta2', category=category)


        self.loader.process()
        created_election = Election.objects.get(name="Algarrobo")
        self.assertTrue(Question.objects.filter(category__election=created).exists())


from django.conf.urls.defaults import *

from tastypie.api import Api
from elections.api import ElectionResource, CandidateResource
from elections.api_v2 import ElectionV2Resource, CategoryV2Resource, QuestionV2Resource,\
							AnswerV2Resource, CandidateV2Resource, PersonalDataCandidateV2Resource,\
							BackgroundsCandidateV2Resource, PersonalDataV2Resource, BackgroundCategoryV2Resource,\
							BackgroundV2Resource, LinkV2Resource, MediaNaranjaResource, InformationSourceResource

v1_api = Api(api_name='v1')
v1_api.register(ElectionResource())
v1_api.register(CandidateResource())

v2_api = Api(api_name='v2')
v2_api.register(ElectionV2Resource())
v2_api.register(CategoryV2Resource())
v2_api.register(QuestionV2Resource())
v2_api.register(AnswerV2Resource())
v2_api.register(CandidateV2Resource())
v2_api.register(PersonalDataCandidateV2Resource())
v2_api.register(BackgroundsCandidateV2Resource())
v2_api.register(PersonalDataV2Resource())
v2_api.register(BackgroundCategoryV2Resource())
v2_api.register(BackgroundV2Resource())
v2_api.register(LinkV2Resource())
v2_api.register(MediaNaranjaResource())
v2_api.register(InformationSourceResource())

urlpatterns = patterns('',
    (r'^', include(v1_api.urls),),
    (r'^', include(v2_api.urls),),
)


from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError

from elections.models import Election, PersonalData, Candidate, PersonalDataCandidate
from elections.forms import PersonalDataForm, PersonalDataCandidateForm


class PersonalDataModelTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

    def test_create_personal_data(self):
        personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')
        self.assertTrue(created)
        self.assertEqual(personal_data.label, 'foo')
        self.assertEqual(personal_data.election, self.election)


class PersonalDataCreateView(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

    def test_create_personal_data_by_user_without_login(self):
        request = self.client.get(reverse('personal_data_create',
                                    kwargs={'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 302)

    def test_create_personal_data_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('personal_data_create',
                                    kwargs={'election_slug': self.election.slug}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], PersonalDataForm))
        self.assertTrue('election' in request.context)
        self.assertTrue(isinstance(request.context['election'], Election))

    def test_post_personal_data_create_without_login(self):
        params = {'label': 'Bar'}
        response = self.client.post(reverse('personal_data_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_personal_data_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('personal_data_create',
                                    kwargs={'election_slug': 'strager_election_slug'}))
        self.assertEquals(response.status_code, 404)

    def test_post_personal_data_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')

        params = {'label': 'Bar'}
        response = self.client.post(reverse('personal_data_create',
                                        kwargs={'election_slug': 'strager_election_slug'}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_personal_data_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'label': 'Bar'}
        response = self.client.post(reverse('personal_data_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = PersonalData.objects.filter(label='Bar')
        self.assertEquals(qs.count(), 1)
        personal_data = qs.get()
        self.assertEqual(personal_data.label, params['label'])
        self.assertEqual(personal_data.election, self.election)

        self.assertRedirects(response, reverse('personal_data_create',
                                               kwargs={'election_slug': self.election.slug}))
                                               
                                               
    def test_step_three_template_rendered(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('personal_data_create',
                                    kwargs={'election_slug': self.election.slug}))
        
        self.assertTemplateUsed(response,'elections/wizard/step_three.html')



class PersonalDataCandidateModelTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')
        self.candidate = Candidate.objects.create(name='Juan Candidato',
                                                            election=self.election)

    def test_personal_data_candidate_create(self):
        personal_data_candidate, created = PersonalDataCandidate.objects.get_or_create(candidate=self.candidate,
                                                                                       personal_data=self.personal_data,
                                                                                       value='new_value')

        self.assertEqual(personal_data_candidate.candidate, self.candidate)
        self.assertEqual(personal_data_candidate.personal_data, self.personal_data)
        self.assertEqual(personal_data_candidate.value, 'new_value')





class PersonalDataCandidateCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')
        self.candidate = Candidate.objects.create(name='Juan Candidato', election=self.election)

    def test_create_personal_data_candidate_by_user_without_login(self):
        request = self.client.get(reverse('personal_data_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'personal_data_pk': self.personal_data.pk}))
        self.assertEquals(request.status_code, 302)

    def test_create_personal_data_candidate_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('personal_data_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'personal_data_pk': self.personal_data.pk}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], PersonalDataCandidateForm))
        self.assertTrue('candidate' in request.context)
        self.assertTrue(isinstance(request.context['candidate'], Candidate))
        self.assertTrue('personal_data' in request.context)
        self.assertTrue(isinstance(request.context['personal_data'], PersonalData))

    def test_post_personal_data_candidate_create_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.post(reverse('personal_data_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'personal_data_pk': self.personal_data.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_personal_data_candidate_create_with_login_stranger_candidate(self):
        self.user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.personal_data2, created = PersonalData.objects.get_or_create(election=self.election2,
                                                                    label='foo')
        self.candidate2 = Candidate.objects.create(name='Juan Candidato', election=self.election2)

        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('personal_data_candidate_create',
                                    kwargs={'candidate_pk': self.candidate2.pk,
                                            'personal_data_pk': self.personal_data2.pk}))
        self.assertEquals(response.status_code, 404)

    def test_post_personal_data_candidate_create_with_login_stranger_candidate(self):
        self.user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.personal_data2, created = PersonalData.objects.get_or_create(election=self.election2,
                                                                    label='foo')
        self.candidate2 = Candidate.objects.create(name='Juan Candidato', election=self.election2)

        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('personal_data_candidate_create',
                                        kwargs={'candidate_pk': self.candidate2.pk,
                                                'personal_data_pk': self.personal_data2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_personal_data_candidate_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('personal_data_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'personal_data_pk': self.personal_data.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.content, '{"value": "%s"}' % params['value'])

        expected = {self.personal_data.label: params['value']}

        self.assertEquals(params['value'], expected[self.personal_data.label])


class AsyncDeletePersonalDataTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')
        self.personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')

    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')

        request = self.client.post(reverse('async_delete_personal_data',
                                kwargs={'personal_data_pk': self.personal_data.pk}),
                                        {})
        self.assertEquals(request.status_code, 200)

    def test_post_without_login(self):
        request = self.client.post(reverse('async_delete_personal_data',
                                kwargs={'personal_data_pk': self.personal_data.pk}),
                                        {})
        self.assertEquals(request.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('async_delete_personal_data',
                                kwargs={'personal_data_pk': self.personal_data.pk}))

        self.assertEquals(request.status_code, 405)

    def test_post_with_stranger_candidate(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        personal_data2, created = PersonalData.objects.get_or_create(election=election2,
                                                                        label='foo')

        self.client.login(username='joe', password='doe')
        request = self.client.post(reverse('async_delete_personal_data',
                                kwargs={'personal_data_pk': personal_data2.pk}))

        self.assertEquals(request.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.personal_data.pk
        request = self.client.post(reverse('async_delete_personal_data',
                                kwargs={'personal_data_pk': self.personal_data.pk}),
                                        {})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(PersonalData.DoesNotExist, PersonalData.objects.get, pk=temp_pk)


class AsyncCreatePersonalDataView(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.personal_data, created = PersonalData.objects.get_or_create(election=self.election,
                                                                    label='foo')

        self.user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.personal_data2, created = PersonalData.objects.get_or_create(election=self.election2,
                                                                    label='foobar')


    def test_get_async_create_personal_data_with_login(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('async_create_personal_data',
                                    kwargs={'election_pk': self.election.pk}))
        self.assertEqual(response.status_code, 405)

    def test_get_async_create_personal_data_without_login(self):
        response = self.client.get(reverse('async_create_personal_data',
                                    kwargs={'election_pk': self.election.pk}))
        self.assertEqual(response.status_code, 302)


    def test_post_async_create_personal_data_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_personal_data',
                                    kwargs={'election_pk': self.election.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_post_async_create_personal_data_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_personal_data',
                                    kwargs={'election_pk': self.election2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_async_create_personal_data_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_personal_data',
                                    kwargs={'election_pk': self.election.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)

        personal_datas = self.election.personaldata_set.all()
        personal_data_labels = [ personal_data.label for personal_data in personal_datas]

        self.assertTrue(params['value'] in personal_data_labels)

        personal_data = self.election.personaldata_set.get(label=params['value'])
        self.assertEquals(response.content, '{"pk": %d, "label": "%s"}' % (personal_data.pk, params['value']))



from django.conf import settings
from django.test import TestCase
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.utils import simplejson

from elections.models import Candidate, Election, Category, Question, Answer
from elections.forms import AnswerForm
from django.core.exceptions import ValidationError

class AnswersTest(TestCase):
    def test_create_answer(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user,
                                                            slug='barbaz')
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=election)
        question, created = Question.objects.get_or_create(question='Foo',
                                                            category=category)
        answer = Answer.objects.create(question=question, caption='Bar')
        self.assertEquals(answer.caption, 'Bar')
        self.assertEquals(answer.question, question)


    def test_cannot_create_empty_answer(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user,
                                                            slug='barbaz')
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=election)
        question, created = Question.objects.get_or_create(question='Foo',
                                                            category=category)
        answer = Answer(question=question, caption='')

        with self.assertRaises(ValidationError) as e:
            answer.full_clean()
            expected_error = {'caption':[u'This field cannot be blank.']}
            self.assertEqual(e.message_dict,expected_error)           


class AnswerCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                                owner=self.user,
                                                                slug='barbaz')
        self.category, created = Category.objects.get_or_create(name='FooCat',
                                                                election=self.election)
        self.question, created = Question.objects.get_or_create(question='Foo',
                                                                category=self.category)

    def test_create_answer_by_user_without_login(self):
        request = self.client.get(reverse('answer_create',
                                    kwargs={'question_pk': self.question.pk}))
        self.assertEquals(request.status_code, 302)

    def test_create_answer_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('answer_create',
                                    kwargs={'question_pk': self.question.pk}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], AnswerForm))
        self.assertTrue('question' in request.context)
        self.assertTrue(isinstance(request.context['question'], Question))

    def test_post_answer_create_without_login(self):
        params = {'caption': 'Bar'}
        response = self.client.post(reverse('answer_create',
                                        kwargs={'question_pk': self.question.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_answer_create_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('answer_create',
                                    kwargs={'question_pk': 97965678765}))
        self.assertEquals(response.status_code, 404)

    def test_post_answer_create_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')

        params = {'caption': 'Bar'}
        response = self.client.post(reverse('answer_create',
                                        kwargs={'question_pk': 23678543567}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_answer_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'caption': 'Bar'}
        response = self.client.post(reverse('answer_create',
                                        kwargs={'question_pk': self.question.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = Answer.objects.filter(caption=params['caption'])
        self.assertEquals(qs.count(), 1)
        answer = qs.get()
        self.assertEqual(answer.caption, params['caption'])
        self.assertEqual(answer.question, self.question)

        self.assertRedirects(response, reverse('category_create',
                                               kwargs={'election_slug': self.election.slug}))


class CreateAnswerWithCategoryAjax(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='joe', email='joe@doe.cl')
        self.user2 = User.objects.create_user(username='doe', password='joe', email='dow@doe.cl')
        self.election = Election.objects.create(name='BarBaz',
            owner=self.user,
            slug='barbaz')
        self.category = Category.objects.create(name='FooCat',
            election=self.election)
        self.question = Question.objects.create(question='Foo',
            category=self.category)

        self.url = reverse('answer_create_ajax', kwargs={'question_pk': self.question.pk})

    def test_post_without_login(self):
        params = {'caption': 'Foo bar'}
        response = self.client.post(self.url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 403)

    def test_post_to_not_owned_election(self):
        self.client.login(username=self.user2.username, password='joe')
        params = {'caption': 'Foo bar'}
        response = self.client.post(self.url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 403)

    def test_non_existing_question(self):
        url = reverse('answer_create_ajax', kwargs={'question_pk': 0})
        self.client.login(username=self.user.username, password='joe')
        params = {'caption': 'Foo bar'}
        response = self.client.post(url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 404)

    def test_post_as_election_owner(self):
        self.client.login(username=self.user.username, password='joe')
        params = {'caption': 'Foo bar'}
        response = self.client.post(self.url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)
        answer = Answer.objects.filter(question=self.question)
        self.assertEqual(answer.count(), 1)
        answer = answer.get()
        self.assertEqual(answer.caption, params['caption'])
        self.assertEqual(simplejson.loads(response.content),
                {'pk': answer.pk, 'caption': answer.caption, 'question': self.question.pk})
        self.assertEqual(response['Content-Type'], 'application/json')

    def test_post_invalid_answer(self):
        self.client.login(username=self.user.username, password='joe')
        params = {}

        response = self.client.post(self.url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        answer = Answer.objects.filter(question=self.question)
        self.assertEqual(answer.count(), 0)

        self.assertTrue('error' in simplejson.loads(response.content))
        self.assertEqual(response['Content-Type'], 'application/json')

    def test_get_no_ajax(self):
        self.client.login(username=self.user.username, password='joe')
        params = {'caption': 'Foo bar'}
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 400)
        response = self.client.post(self.url, params)
        self.assertEqual(response.status_code, 400)

    def test_post_empty_answer(self):
        self.client.login(username=self.user.username, password='joe')
        params = {'caption': ''}

        response = self.client.post(self.url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        answer = Answer.objects.filter(question=self.question)
        self.assertEqual(answer.count(), 0)

        self.assertTrue('error' in simplejson.loads(response.content))
        expected_error = {u'error':{u'caption':[u'This field is required.']}}
        self.assertEqual(simplejson.loads(response.content), expected_error)
        self.assertEqual(response['Content-Type'], 'application/json')


class AsyncDeleteAnswerTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='joe', email='joe@doe.cl')
        self.user2 = User.objects.create_user(username='doe', password='joe', email='dow@doe.cl')
        self.election = Election.objects.create(name='BarBaz',
            owner=self.user,
            slug='barbaz')
        self.category = Category.objects.create(name='FooCat',
            election=self.election)
        self.question = Question.objects.create(question='Foo',
            category=self.category)
        self.answer = Answer.objects.create(caption='The first answer',
                question = self.question)

        self.url = reverse('answer_delete_ajax', kwargs={'pk':
            self.answer.pk})
        
        self.assertEqual(Answer.objects.filter(pk=self.answer.pk).count(),1)


    def test_delete_correctly_an_answer(self):
        previous_count = Answer.objects.all().count()
        self.client.login(username=self.user.username, password='joe')
        response = self.client.post(self.url)
        count_after_deletion_of_one_answer = Answer.objects.all().count()
        self.assertEqual(Answer.objects.filter(pk=self.answer.pk).count(),0)
        self.assertEqual(count_after_deletion_of_one_answer, previous_count - 1)

    def test_delete_only_by_owner(self):
        self.client.login(username=self.user2.username, password='joe')
        response = self.client.post(self.url)
        self.assertEqual(response.status_code, 404)

    def test_if_answer_does_not_exist_get_404(self):
        self.client.login(username=self.user.username, password='joe')
        unexistentid = 199999
        url = reverse('answer_delete_ajax', kwargs={'pk': \
                unexistentid})
        response = self.client.post(url)
        self.assertEqual(response.status_code, 404)



# -*- coding: utf-8 -*-
from tastypie.resources import ModelResource, ALL, ALL_WITH_RELATIONS, Resource
from elections.models import Election, Category, Question, Answer, Candidate, PersonalData,\
                            PersonalDataCandidate, Link, Background, BackgroundCandidate, BackgroundCategory,\
                            InformationSource
from tastypie.authentication import ApiKeyAuthentication
from tastypie import fields
from tastypie.serializers import Serializer

class ElectionV2Resource(ModelResource):
    candidates = fields.ToManyField('candidator.elections.api_v2.CandidateV2Resource', 'candidate_set', null=True)
    categories = fields.ToManyField('candidator.elections.api_v2.CategoryV2Resource', 'category_set', null=True)
    background_categories = fields.ToManyField('candidator.elections.api_v2.BackgroundCategoryV2Resource', 'backgroundcategory_set', null=True)
    personal_data = fields.ToManyField('candidator.elections.api_v2.PersonalDataV2Resource', 'personaldata_set', null=True)
    
    class Meta:
        queryset = Election.objects.all()
        resource_name = 'election'
        authentication = ApiKeyAuthentication()

    def authorized_read_list(self, object_list, bundle):
        return object_list.filter(owner=bundle.request.user)

class PersonalDataV2Resource(ModelResource):
    class Meta:
        queryset = PersonalData.objects.all()
        resource_name = 'personal_data'
        authentication = ApiKeyAuthentication()

class LinkV2Resource(ModelResource):
    class Meta:
        queryset = Link.objects.all()
        resource_name = 'link'
        authentication = ApiKeyAuthentication()

class BackgroundV2Resource(ModelResource):
    background_category = fields.ToOneField('candidator.elections.api_v2.BackgroundCategoryV2Resource','category')

    class Meta:
        queryset = Background.objects.all()
        resource_name = 'background'
        authentication = ApiKeyAuthentication()

class PersonalDataCandidateV2Resource(ModelResource):
    candidate = fields.ToOneField('candidator.elections.api_v2.CandidateV2Resource','candidate')
    personal_data = fields.ToOneField('candidator.elections.api_v2.PersonalDataV2Resource','personal_data')

    class Meta:
        resource_name = 'personal_data_candidate'
        queryset = PersonalDataCandidate.objects.all()
        authentication = ApiKeyAuthentication()

class BackgroundsCandidateV2Resource(ModelResource):
    background = fields.ToOneField('candidator.elections.api_v2.BackgroundV2Resource','background')

    class Meta:
        resource_name = 'backgrounds_candidate'
        queryset = BackgroundCandidate.objects.all()

class CandidateV2Resource(ModelResource):
    # personal_data = fields.ManyToManyField(PersonalDataV2Resource, 'personal_data', null=True, full=True)
    links = fields.ToManyField(LinkV2Resource, 'link_set', null=True)
    personal_data_candidate = fields.ToManyField('candidator.elections.api_v2.PersonalDataCandidateV2Resource','personaldatacandidate_set', null=True)
    backgrounds_candidate = fields.ToManyField('candidator.elections.api_v2.BackgroundsCandidateV2Resource','backgroundcandidate_set', null=True)
    answers = fields.ToManyField('candidator.elections.api_v2.AnswerV2Resource','answers', null=True)

    class Meta:
        queryset = Candidate.objects.all()
        resource_name = 'candidate'
        authentication = ApiKeyAuthentication()
        filtering = {
            'id' : ALL
        }

class AnswerV2Resource(ModelResource):
    question = fields.ToOneField('candidator.elections.api_v2.QuestionV2Resource', 'question')
    candidates = fields.ToManyField(CandidateV2Resource, 'candidate_set', null=True)

    class Meta:
        queryset = Answer.objects.all()
        resource_name = 'answer'
        authentication = ApiKeyAuthentication()
        filtering = {
            'question' : ALL,
            'candidate' : ALL
        }

class QuestionV2Resource(ModelResource):
    category = fields.ToOneField('candidator.elections.api_v2.CategoryV2Resource', 'category', null=True)
    answers = fields.ToManyField(AnswerV2Resource, 'answer_set', null=True)
    information_sources = fields.ToManyField('candidator.elections.api_v2.InformationSourceResource', 'informationsource_set', null=True)

    class Meta:
        queryset = Question.objects.all()
        resource_name = 'question'
        authentication = ApiKeyAuthentication()

class CategoryV2Resource(ModelResource):
    questions = fields.ToManyField(QuestionV2Resource, 'question_set', null=True)

    class Meta:
        queryset = Category.objects.all()
        resource_name = 'category'
        authentication = ApiKeyAuthentication()

class BackgroundCategoryV2Resource(ModelResource):
    background = fields.ToManyField('candidator.elections.api_v2.BackgroundV2Resource', 'background_set', null=True)

    class Meta:
        queryset = BackgroundCategory.objects.all()
        resource_name = 'background_category'
        authentication = ApiKeyAuthentication()

from candidator.elections.views import strip_elements_from_dictionary, medianaranja2

class MediaNaranjaResource(Resource):
    class Meta:
        resource_name = 'medianaranja'
        object_class = dict
        always_return_data = True
        serializer = Serializer(formats=['jsonp', 'json'])

    def obj_create(self,bundle,**kwargs):
        election = Election.objects.get(id=bundle.data["election-id"])

        elements = strip_elements_from_dictionary(bundle.data["data"],election)
        result = medianaranja2(elements["answers"], elements['importances'], elements['questions'], elements['candidates'], elements['categories'], election)
        bundle.obj = result
        bundle.data = result
        return bundle

    def detail_uri_kwargs(self, bundle):
        return {}

    def serialize(self, request, data, format, options=None):
        winner = data.data["winner"]
        others = data.data["others"]
        data.data["winner"] = {
                'global_score':winner[0], 
                'category_score':winner[1], 
                'candidate':winner[2].id
                }
        categories = data.data["categories"]
        counter = 0
        scores = []
        for score  in data.data["winner"]["category_score"]:
            category_dict = {
             "category":categories[counter].name,
             "score":score
            }
            scores.append(category_dict)
            counter += 1
        data.data["winner"]["category_score"] = scores

        others_counter = 0
        for other in others:
            others_dict = {
                'global_score':other[0], 
                'category_score':other[1], 
                'candidate':other[2].id
            }
            score_counter = 0
            scores = []
            for score in others_dict["category_score"]:
                category_dict = {
                 "category":categories[score_counter].name,
                 "score":score
                }
                scores.append(category_dict)

                score_counter += 1

            others_dict["category_score"] = scores



            data.data["others"][others_counter] = others_dict
            others_counter += 1

        serialized = super(MediaNaranjaResource, self).serialize(request, data, format, options)

        return serialized

class InformationSourceResource(ModelResource):
    candidate = fields.ToOneField('candidator.elections.api_v2.CandidateV2Resource','candidate')
    question = fields.ToOneField('candidator.elections.api_v2.QuestionV2Resource', 'question')


    class Meta:
        queryset = InformationSource.objects.all()
        resource_name = 'information_source'
        authentication = ApiKeyAuthentication()


# Create your views here.
from django.views.generic import CreateView
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.http import HttpResponse, Http404

from forms import ReportForm
from models import Report


class ReportCreateView(CreateView):
    model = Report
    form_class = ReportForm

    def dispatch(self, request, *args, **kwargs):
        self.report_object = self._get_report_object(**kwargs)
        return super(ReportCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, *args, **kwargs):
        context = super(ReportCreateView, self).get_context_data(*args, **kwargs)
        context['report_object'] = self.report_object
        return context

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.content_object = self.report_object
        self.object.save()
        return redirect(self.get_success_url())

    def _get_report_object(self, **kwargs):
        try:
            model_class = ContentType.objects.get(pk=kwargs['content_type']).model_class()
            self.report_object = get_object_or_404(model_class, pk=kwargs['object_id'])
        except ContentType.DoesNotExist:
            raise Http404
        return self.report_object




# encoding=UTF-8
from django.test import TestCase
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.template import Template, Context
from django.utils.translation import ugettext as _

from elections.models import Candidate, Election, Category, Question, Answer
from elections.forms import CategoryForm, ElectionForm


class CandidateTagsTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='doe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.not_user = User.objects.create_user(username='joe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.election2, created = Election.objects.get_or_create(name='BarBaz2',
                                                            owner=self.user,
                                                            slug='barbaz2')
        self.candidate = Candidate.objects.create(name='Bar Baz',
                                                            election=self.election)
        self.candidate2, created = Candidate.objects.get_or_create(
                                                            name='Bar Baz',
                                                            election=self.election2)
        categories = [
            Category.objects.get_or_create(election=self.election,
                                            name='Cat1',
                                            slug='cat1'),
            Category.objects.get_or_create(election=self.election,
                                            name='Cat2',
                                            slug='cat2'),
        ]
        self.categories = [cat for cat, created in categories]
        category2, created = Category.objects.get_or_create(election=self.election2,
                                                     name='Cat2')
        question, created = Question.objects.get_or_create(question='Foo',
                                                            category=self.categories[0])
        question2, created = Question.objects.get_or_create(question='Foo',
                                                            category=category2)
        self.answer, created = Answer.objects.get_or_create(question=question,
                                                       caption='Bar')
        self.answer2, created = Answer.objects.get_or_create(question=question2,
                                                       caption='Bar')

    def test_answer_for_candidate_and_question(self):
        self.candidate.associate_answer(self.answer)
        template = Template('{% load election_tags %}{% answer_for_candidate_and_question candidate question %}')
        context = Context({"candidate": self.candidate, "question": self.answer.question})
        self.assertEqual(template.render(context), self.answer.caption)

    def test_answer_for_candidate_and_question_no_caption(self):
        template = Template('{% load election_tags %}{% answer_for_candidate_and_question candidate question %}')
        context = Context({"candidate": self.candidate, "question": self.answer.question})
        self.assertEqual(template.render(context), _(u'Aún no hay respuesta'))


        

class CandidateAnswerTest(TestCase):
    def setUp(self):
        #TODO: make pretty
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user,
                                                            slug='barbaz')
        candidate, created = Candidate.objects.get_or_create(name='Bar Baz',
                                                            election=election)
        category, created = Category.objects.get_or_create(name='FooCat',
                                                            election=election)
        question, created = Question.objects.get_or_create(question='Foo',
                                                            category=category)
        question2, created = Question.objects.get_or_create(question='Bar',
                                                            category=category)
        answer, created = Answer.objects.get_or_create(question=question,
                                                        caption='Bar')
        answer2, created = Answer.objects.get_or_create(question=question,
                                                        caption='Bar')
        answer3, created = Answer.objects.get_or_create(question=question2,
                                                        caption='Bar')
        self.user = user
        self.election = election
        self.candidate = candidate
        self.category = category
        self.question = question
        self.question2 = question2
        self.answer = answer
        self.answer2 = answer2
        self.answer3 = answer3

    def test_associate_candidate_answer(self):
        self.candidate.associate_answer(self.answer)
        self.assertEquals(list(self.candidate.answers.all()), [self.answer])

    def test_associate_candidate_answer_same_question(self):
        self.candidate.associate_answer(self.answer)
        self.candidate.associate_answer(self.answer2)
        self.assertEquals(list(self.candidate.answers.all()), [self.answer2])

    def test_associate_candidate_second_answer(self):
        self.candidate.associate_answer(self.answer2)
        self.candidate.associate_answer(self.answer3)
        self.assertEquals(list(self.candidate.answers.all()),
                            [self.answer2, self.answer3])


class AssociateCandidatesAnswersTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='doe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.not_user = User.objects.create_user(username='joe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.election2, created = Election.objects.get_or_create(name='BarBaz2',
                                                            owner=self.user,
                                                            slug='barbaz2')
        #deleting default categories
        for category in self.election.category_set.all():
            category.delete()
        for category in self.election2.category_set.all():
            category.delete()
        #end of deleting default categories
        self.candidate = Candidate.objects.create(name='Bar Baz',
                                                            election=self.election)
        self.candidate2, created = Candidate.objects.get_or_create(
                                                            name='Bar Baz',
                                                            election=self.election2)
        categories = [
            Category.objects.get_or_create(election=self.election,
                                            name='Cat1',
                                            slug='cat1'),
            Category.objects.get_or_create(election=self.election,
                                            name='Cat2',
                                            slug='cat2'),
        ]
        self.categories = [cat for cat, created in categories]
        category2, created = Category.objects.get_or_create(election=self.election2,
                                                     name='Cat2')
        question, created = Question.objects.get_or_create(question='Foo',
                                                            category=self.categories[0])
        question2, created = Question.objects.get_or_create(question='Foo',
                                                            category=category2)
        self.answer, created = Answer.objects.get_or_create(question=question,
                                                       caption='Bar')
        self.answer2, created = Answer.objects.get_or_create(question=question2,
                                                       caption='Bar')

    def test_get_associate_answers_to_candidate_view(self):
        # GET without login
        request = self.client.get(reverse('associate_answer_candidate',
                                            kwargs={'candidate_slug': self.candidate2.slug,
                                                    'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 302)

        # GET of non existing candidate/user pair associated with the logged in user
        self.client.login(username='joe', password='joe')
        request = self.client.get(reverse('associate_answer_candidate',
                                            kwargs={'candidate_slug': self.candidate2.slug,
                                                    'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 404)

        # GET of existing candidate/user pair associated with the logged in user
        self.client.login(username='doe', password='joe')
        request=self.client.get(reverse('associate_answer_candidate',
                                            kwargs={'candidate_slug': self.candidate.slug,
                                                    'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 200)

        self.assertTrue('candidate' in request.context)
        self.assertEquals(request.context['candidate'], self.candidate)

        self.assertTrue('categories' in request.context)
        self.assertEquals(list(request.context['categories'].all()), self.categories)

    def test_post_associate_answer_to_candidate_view(self):
        request = self.client.post(reverse('associate_answer_candidate',
                                            kwargs={'candidate_slug': self.candidate2.slug,
                                                    'election_slug': self.election.slug}),
                                 {'answer': self.answer.pk})
        self.assertEquals(request.status_code, 302)

        self.client.login(username='joe', password='joe')
        request = self.client.post(reverse('associate_answer_candidate',
                                            kwargs={'candidate_slug': self.candidate2.slug,
                                                     'election_slug': self.election.slug}),
                                 {'answer': self.answer.pk})
        self.assertEquals(request.status_code, 404)

        self.client.login(username='doe', password='joe')
        request=self.client.post(reverse('associate_answer_candidate',
                                            kwargs={'candidate_slug': self.candidate.slug,
                                                    'election_slug': self.election.slug}),
                                 {'answer': self.answer.pk})
        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"answer": %d}' % self.answer.pk)
        self.assertEquals(list(self.candidate.answers.all()), [self.answer])

        self.client.login(username='doe', password='joe')
        request=self.client.post(reverse('associate_answer_candidate',
                                         kwargs={'candidate_slug': self.candidate.slug,
                                                 'election_slug': self.election.slug}),
                                 {'answer': self.answer2.pk})
        self.assertEquals(request.status_code, 404)





from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView, ListView, TemplateView, RedirectView
from django.contrib.sites.models import Site
from django.views.decorators.http import require_POST


# Import forms
from elections.forms import ElectionForm, ElectionUpdateForm, PersonalDataForm,\
                            BackgroundCategoryForm, BackgroundForm, QuestionForm, \
                            CategoryForm, ElectionStyleUpdateForm

# Import models
from elections.forms.candidate_form import CandidateForm
from elections.forms.election_form import AnswerForm, ElectionLogoUpdateForm
from elections.models import Election, Candidate, Category

from django.conf import settings

# Election Views
class ElectionLogoUpdateView(UpdateView):
    model = Election
    form_class = ElectionLogoUpdateForm
    
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.election = get_object_or_404(Election, pk=kwargs['pk'], owner=request.user)
        return super(ElectionLogoUpdateView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(ElectionLogoUpdateView, self).get_form_kwargs()
        kwargs['election'] = self.object
        return kwargs
    
    def get_template_names(self):
        return 'elections/updating/election_logo_form.html'

    def get_success_url(self):
        url = reverse('election_update', kwargs={'slug': self.object.slug})
        return url

class ElectionStyleUpdateView(UpdateView):
    model = Election
    form_class = ElectionStyleUpdateForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.election = get_object_or_404(Election, slug=kwargs['slug'], owner=request.user)
        return super(ElectionStyleUpdateView, self).dispatch(request, *args, **kwargs)

    def get_template_names(self):
        return 'elections/updating/election_style_updating.html'

    def get_queryset(self):
        return super(ElectionStyleUpdateView, self).get_queryset().filter(owner=self.request.user)

    def get_success_url(self):
        url = reverse('update_custom_style', kwargs={'slug': self.object.slug})
        return url


class ElectionUpdateView(UpdateView):
    model = Election
    form_class = ElectionUpdateForm

    def get_template_names(self):
        return 'elections/updating/election_basic_information.html'

    def get_success_url(self):
        return reverse('election_update', kwargs={'slug': self.object.slug})

    def get_context_data(self, **kwargs):
        context = super(ElectionUpdateView, self).get_context_data(**kwargs)
        election = kwargs['form'].instance
        context['election_url'] = self.request.build_absolute_uri(election.get_absolute_url())
        new_candidate_form = CandidateForm()
        context['new_candidate_form'] = new_candidate_form
        return context

    def get_queryset(self):
        return super(ElectionUpdateView, self).get_queryset().filter(owner=self.request.user)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        election = get_object_or_404(Election, slug=kwargs['slug'], owner=request.user)
        return super(ElectionUpdateView, self).dispatch(request, *args, **kwargs)


# Election views
class ElectionDetailView(DetailView):
    model = Election

    def get_queryset(self):

        return super(ElectionDetailView, self).get_queryset().filter(owner__username=self.kwargs['username']).filter(published=True)

class ElectionShareView(DetailView):
    model = Election
    def get_queryset(self):
        return super(ElectionShareView, self).get_queryset().filter(owner=self.request.user)

    def get_context_data(self, **kwargs):
            self.object.published=True
            self.object.save()
            context = super(ElectionShareView, self).get_context_data(**kwargs)
            return context
    # def get_queryset(self):
    #     return super(ElectionDetailView, self).get_queryset().filter(owner__username=self.kwargs['username']).filter(published=True)



class ElectionCreateView(CreateView):
    model = Election
    form_class = ElectionForm
    
    def get_template_names(self):
        return ['elections/wizard/step_one.html']

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(ElectionCreateView, self).dispatch(request, *args, **kwargs)

    def get_success_url(self):
        return reverse('candidate_create', kwargs={'election_slug': self.object.slug})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.owner = self.request.user
        self.object.set_slug()
        self.object.full_clean()

        return super(ElectionCreateView, self).form_valid(form)

# Election views
class CompareView(DetailView):
    model = Election

    def get_queryset(self):
        return super(CompareView, self).get_queryset().filter(owner__username=self.kwargs['username']).filter(published=True)


    def get_context_data(self, **kwargs):
        context = super(CompareView, self).get_context_data(**kwargs)
        username = self.kwargs['username']
        slug = self.kwargs['slug']
        #Seleccionar esto del get_queryset
        #con algo como self.object
        election = get_object_or_404(Election, owner__username=username, slug=slug)
        context["election"] = election

        if('first_candidate_slug' in self.kwargs):

            first_candidate_slug = self.kwargs['first_candidate_slug']

            first_candidate = get_object_or_404(Candidate, election=election, slug=first_candidate_slug)
            context['first_candidate'] = first_candidate
            if('second_candidate_slug' in self.kwargs):
                second_candidate_slug = self.kwargs['second_candidate_slug']
                second_candidate = get_object_or_404(Candidate, election=election, slug=second_candidate_slug)
                context['second_candidate'] = second_candidate

                if first_candidate == second_candidate:
                    raise Http404
                if 'category_slug' in self.kwargs:
                    category_slug = self.kwargs['category_slug']
                    selected_category = get_object_or_404(Category, election=election, slug=category_slug)
                    first_candidate_answers = first_candidate.get_all_answers_by_category(selected_category)
                    second_candidate_answers = second_candidate.get_all_answers_by_category(selected_category)
                    answers = first_candidate.get_answers_two_candidates(second_candidate,selected_category)
                    context['selected_category'] = selected_category
                    context['answers'] = answers
                else:
                    return context

            else:
                return context
        else:
            return context
        facebook_link = 'http'
        site = Site.objects.get_current()
        if self.request.is_secure(): facebook_link += 's'
        facebook_link += '://' + site.domain + '/' + username + '/' + slug + '/compare/'
        facebook_link += min(first_candidate_slug,second_candidate_slug) + '/' + max(first_candidate_slug,second_candidate_slug)+ '/' + category_slug
        
        
        
        context['facebook_link'] = facebook_link
        return context

@require_POST
def election_compare_asynchronous_call(request, username, slug, candidate_slug):
    election = get_object_or_404(Election, slug=slug, owner__username=username)
    candidate = get_object_or_404(Candidate, slug=candidate_slug, election=election)
    personal_data = candidate.get_personal_data
    try:
        photo_route = str(candidate.photo.url)
    except :
        photo_route = 'media/photos/dummy.jpg'
    json_dictionary = {"personal_data":personal_data,"photo_route":photo_route}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')




# Election views
class ElectionAboutView(DetailView):
    model = Election
    template_name = "elections/election_about.html"
    def get_template_names(self):
        return [self.template_name]

    def get_queryset(self):

        return super(ElectionAboutView, self).get_queryset().filter(owner__username=self.kwargs['username']).filter(published=True)


class PrePersonalDataView(TemplateView):

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.election = get_object_or_404(Election, slug=kwargs['election_slug'], owner=request.user)
        return super(PrePersonalDataView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(PrePersonalDataView, self).get_context_data(**kwargs)
        context['election'] = self.election
        return context

class ElectionUpdateDataView(DetailView):
    model = Election
    def get_template_names(self):
        return ['elections/updating/questions.html']

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(ElectionUpdateDataView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ElectionUpdateDataView, self).get_context_data(**kwargs)
        context['personaldata_form'] = PersonalDataForm()
        context['backgroundcategory_form'] = BackgroundCategoryForm()
        context['background_form'] = BackgroundForm()
        context['question_form'] = QuestionForm(election=self.object)
        context['category_form'] = CategoryForm()
        context['answer_form'] = AnswerForm()
        return context

    def get_queryset(self):
        return super(ElectionUpdateDataView, self).get_queryset().filter(owner=self.request.user)

class ElectionRedirectView(RedirectView):
    permanent = False

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(ElectionRedirectView, self).dispatch(request, *args, **kwargs)

    def get_redirect_url(self, **kwargs):
        num_elections = self.request.user.election_set.count()
        if self.request.user.election_set.count() <= 0:
            return reverse('election_create')
        last_election = self.request.user.election_set.latest('pk')
        if last_election.candidate_set.count() <= 0:
            return reverse('election_detail_admin',
                           kwargs={'slug': last_election.slug, 'username': self.request.user.username})
        return reverse('candidate_data_update',
                       kwargs={'election_slug': last_election.slug, 'slug': last_election.candidate_set.all()[0].slug})


class HomeTemplateView(TemplateView):
    template_name = 'home.html'


    def get_context_data(self,**kwargs):
        last_five_elections = Election.objects.filter(published=True).order_by('-created_at')[:5]
        kwargs['last_elections'] = last_five_elections
        kwargs['highlighted_elections'] = Election.objects.filter(highlighted=True).filter(published=True).order_by('?')[:5]

        return kwargs

class UserElectionsView(TemplateView):
    template_name = 'elections/users_election_list.html'

    def get_context_data(self, **kwargs):
        user = get_object_or_404(User, username=kwargs['username'])
        user_elections = Election.objects.filter(published=True).filter(owner=user)
        context = super(UserElectionsView, self).get_context_data(**kwargs)
        context['elections'] = user_elections

        context['owner'] = user


        return context

##THIS IS ONLY FOR TESTING PORPOUSES

class EmbededTemplateView(TemplateView):
    template_name = 'prueba.html'

    def get_context_data(self, **kwargs):
        return {'embeded_test_web':settings.EMBEDED_TEST_WEB}

class TogglePublishView(UpdateView):
    model = Election
    http_method_names = ['post']

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated():
            http_response = HttpResponse('Unauthorized', {}, {})
            http_response.status_code = 401
            return http_response
        return super(TogglePublishView, self).dispatch(request, *args, **kwargs)

    def get_template_names(self):
        return []

    def post(self, request, *args, **kwargs):
        election = self.get_object()
        if election.owner != request.user:
            http_response = HttpResponse('Forbidden', {}, {})
            http_response.status_code = 403
            return http_response
        election.published = not election.published
        election.save()
        response_data = {}
        response_data['published'] = election.published
        response_data['id'] = election.id
        return HttpResponse(json.dumps(response_data), content_type='application/json')


from django.contrib import admin
from candideitorg.models import Election as CanElection, Candidate, Category, Question, Answer, InformationSource
from candideitorg.tasks import election_updater

class CandidateInline(admin.TabularInline):
    model = Candidate
    fields = ['name', 'photo','has_answered',]
    # fields = ['name', 'photo', 'answers']
    extra = 0

class CanElectionAdmin(admin.ModelAdmin):
    inlines = [
        CandidateInline
    ]
    fields = ['name','description','information_source',   ]
    actions = ['update_election_from_candideit']
    def update_election_from_candideit(self, request, queryset):
        for election in queryset:
            election_updater.delay(election)
    update_election_from_candideit.short_description = "Actualizar elecciones desde candideitorg"
    
admin.site.register(CanElection, CanElectionAdmin)



class CategoryAdmin(admin.ModelAdmin):
    pass    

admin.site.register(Category, CategoryAdmin)


class AnswerInline(admin.TabularInline):
    model = Answer
    extra = 0
    fields = ['caption', ]

class InformationSourceAdmin(admin.TabularInline):
    model = InformationSource
    extra = 0

class QuestionAdmin(admin.ModelAdmin):
    fields = ['question', 'category']
    inlines = [
        AnswerInline,
        InformationSourceAdmin
    ]

admin.site.register(Question, QuestionAdmin)



# coding= utf-8
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from elections.models import Election

class ElectionDestructor(object):
	def __init__(self,username):
		self.user = User.objects.get(username=username)
		self.elections = Election.objects.filter(owner=self.user)

	def process(self):
		for election in self.elections:
			election.personaldata_set.all().delete()
			election.backgroundcategory_set.all().delete()
			election.category_set.all().delete()
		self.elections.delete()

class Command(BaseCommand):
	args = '<username>'
	def handle(self, *args, **options):
		username = args[0]
		destructor = ElectionDestructor(username)
		destructor.process()


from django.test import TestCase
from django.contrib.auth.models import User
from tastypie.models import ApiKey


class UserApiKeyCreationTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')


    def test_exit_api_key_to_a_new_user(self):
        api_key = ApiKey.objects.filter(user = self.user)
        self.assertEqual(len(api_key), 1)



from requests.auth import AuthBase

class ApiKeyAuth(AuthBase):
    """
    Sets the appropriate authentication headers
    for the Tastypie API key authentication.
    """
    def __init__(self, username, api_key):
        self.username = username
        self.api_key = api_key

    def __call__(self, r):
        r.headers['Authorization'] = 'ApiKey %s:%s' % (self.username, self.api_key)
        return r


from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods, require_POST
from django.views.generic import CreateView, DetailView, UpdateView

# Import forms
from elections.forms import QuestionForm

# Import models
from elections.models import Category, Question, Election, Answer


# Question Views
class QuestionCreateView(CreateView):
    model = Question
    form_class = QuestionForm
    
    def get_template_names(self):
        return ['elections/wizard/step_five.html']

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.election = get_object_or_404(Election, slug=kwargs['election_slug'], owner=request.user)
        return super(QuestionCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(QuestionCreateView, self).get_context_data(**kwargs)
        context['election'] = self.election
        return context

    def get_form_kwargs(self, *args, **kwargs):
        kwargs = super(QuestionCreateView, self).get_form_kwargs(*args, **kwargs)
        kwargs['election'] = self.election
        return kwargs

    def get_success_url(self):
        return reverse('question_create', kwargs={'election_slug': self.object.category.election.slug})


@login_required
@require_POST
def async_delete_question(request, question_pk):
    question = get_object_or_404(Question, pk = question_pk, category__election__owner = request.user)
    question.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')


@login_required
@require_http_methods(['POST'])
def async_create_question(request, category_pk):
    pass
    category = get_object_or_404(Category, pk=category_pk, election__owner=request.user)

    value = request.POST.get('value', None)
    question = Question(question=value, category=category)

    try:
        
        question.full_clean()
        question.save()
        return HttpResponse(json.dumps({"pk": question.pk, "question": question.question}),
                        content_type='application/json')
    except Exception as e:
        errors = {
            'error':e.message_dict
        }
        return HttpResponse(json.dumps(errors),
                        content_type='application/json')
        
    


# class QuestionCreateAjaxView(CreateView):
#     model = Question
#     form_class = QuestionForm

#     @method_decorator(login_required)
#     @method_decorator(require_POST)
#     def dispatch(self, request, *args, **kwargs):
#         if not request.is_ajax():
#             return HttpResponseBadRequest()
#         self.category = get_object_or_404(Category, pk=kwargs['category_pk'], election__owner=request.user)
#         if self.category.election.owner != request.user:
#             return HttpResponseForbidden()
#         return super(QuestionCreateAjaxView, self).dispatch(request, *args, **kwargs)

#     def form_valid(self, form):
#         self.object = form.save(commit=False)
#         self.object.category = self.category
#         self.object.save()
#         return HttpResponse(content=json.dumps({"pk": question.pk, "question": question.question}),
#                 content_type='application/json')

#     def form_invalid(self, form):

#         return HttpResponse(content=json.dumps(
#             {'error': form.errors}),
#             content_type='application/json')

#     def get_context_data(self, **kwargs):
#         context = super(QuestionCreateAjaxView, self).get_context_data(**kwargs)
#         context['election'] = self.category.election
#         return context

#     def get_form_kwargs(self, *args, **kwargs):
#         self.object = Question(question=self.request.POST['value'],category=self.category)
#         print(self.object.category)
#         kwargs = super(QuestionCreateAjaxView, self).get_form_kwargs(*args, **kwargs)
#         kwargs['election'] = self.category.election

        
#         return kwargs


# serializers/json_pretty.py
"""
Add the line to settings.py::

    SERIALIZATION_MODULES = {'json-pretty': 'serializers.json_pretty'}

And call dumpdata as follows::

    ./manage.py dumpdata --format=json-pretty <app_name>

"""

import codecs
from django.utils import simplejson
from django.core.serializers.json import Serializer as JSONSerializer
from django.core.serializers.json import Deserializer
from django.core.serializers.json import DjangoJSONEncoder

class Serializer(JSONSerializer):
    def end_serialization(self):
        stream = codecs.getwriter('utf8')(self.stream)
        simplejson.dump(self.objects, stream, cls=DjangoJSONEncoder,
                        ensure_ascii=False, **self.options)
